@@ -1,5 +1,5 @@
 """
-An API and a CLI for quickly building complex canvases.
+An API and a CLI for quickly building complex figures.
 """
 
 __license__ = '''\
@@ -24,13 +24,17 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 '''
 
-usage=r"""
-Usage: %prog [options] [style_config.py] file1.root file2.root ...
-Usage: %prog [options] [style_config.py] file1.root hist1 hist2 ...
-Usage: %prog [options] [style_config.py] file1.root folder1 folder2 ...
+usage="""\
+Usage: %prog [config.py] targets [options]
 
-Documentation: http://packages.python.org/rootplot/"""
-usage = usage[1:]  # remove first (empty) line
+Targets may be either multiple root files to compare or a single root file 
+followed by multiple histograms or folders to compare.  For example:
+    %prog fileA.root fileB.root fileC.root
+    %prog file.root dirA dirB dirC
+    %prog file.root dirA/hist1 dirA/hist2
+
+Full documentation is available at: 
+    http://packages.python.org/rootplot/"""
 
 ##############################################################################
 ######## Import python libraries #############################################
@@ -43,18 +47,31 @@ import os
 import re
 import tempfile
 import copy
-from version import __version__
 from utilities import RootFile, Hist, Hist2D, HistStack, find_num_processors
 from os.path import join as joined
 
+
+##############################################################################
+######## Define globals ######################################################
+
+from version import __version__          # version number
+prog = os.path.basename(sys.argv[0])     # rootplot or rootplotmpl
+use_mpl = False                          # set in plotmpl or rootplotmpl
+global_opts = ['filenames', 'targets', 'debug', 'path', 'processors', 
+               'merge', 'noclean', 'output', 'numbering', 'html_template',
+               'ncolumns_html']
 try:
-    import multiprocessing as multi
+    import multiprocessing
     use_multiprocessing = True
 except ImportError:
     use_multiprocessing = False
 
-## Import ROOT, but we need to temporarily change sys.argv so that ROOT
-## doesn't intercept command-line options
+
+##############################################################################
+######## Import ROOT #########################################################
+
+## We need to temporarily change sys.argv so that ROOT doesn't intercept 
+## options from the command-line
 saved_argv = sys.argv[:]
 argstring = ' '.join(sys.argv)
 sys.argv = [sys.argv[0]]
@@ -74,56 +91,64 @@ up by running 'cmsenv'."""
 ## There is a bug in pyROOT that fails to export colors in batch mode
 if not (re.search('--ext[ =]*C', argstring) or re.search('-e[ ]*C', argstring)):
     ROOT.gROOT.SetBatch()
-ROOT.TH1.SetDefaultSumw2()
 ROOT.gErrorIgnoreLevel = ROOT.kWarning
 if os.path.exists('rootlogon.C'):
     ROOT.gROOT.Macro('rootlogon.C')
-elif os.path.exists(os.path.expanduser('~/.rootlogon.C')):
-    ROOT.gROOT.Macro(os.path.expanduser('~/.rootlogon.C'))
 sys.argv = saved_argv[:]
 
-## Global constants
-prog = os.path.basename(sys.argv[0])
 
 ##############################################################################
 ######## Classes #############################################################
 
 class Options(dict):
-    def __init__(self, options):
+    def __init__(self, options, arguments, scope='global'):
         for opt in dir(options):
             value = getattr(options, opt)
             if (not opt.startswith('__') and
                 type(value) in [int, float, str, bool, type(None)]):
                 self[opt] = value
+        self.filenames = [x for x in arguments if x.endswith('.root')]
+        self.configs   = [x for x in arguments if x.endswith('.py')]
+        self.targets   = [x for x in arguments if not (x.endswith('.py') or
+                                                       x.endswith('.root'))]
+        self.process_configs(scope)
     def __setattr__(self, key, value):
         self[key] = value
     def __getattr__(self, key):
         return self[key]
+    def clean_targets(self):
+        for i in range(len(self.targets)):
+            if self.targets[i][-1] == '/':
+                self.targets[i] = self.targets[i][:-1]
     def arguments(self):
         return self.filenames + self.targets + self.configs
     def kwarg_list(self):
         diffs = {}
-        defaults = parse_arguments([], self.mpl)
+        defaults = parse_arguments([])
         for key, value in self.items():
             if (key not in ['filenames', 'targets', 'configs'] and
                 defaults[key] != value):
                 diffs[key] = value
         return diffs
-    def append_from_package(self, package):
+    def append_from_package(self, package, scope):
         for attribute in dir(package):
             if '__' not in attribute:
-                value = getattr(package, attribute)
-                self[attribute] = value
-    def process_configs(self):
-        myfilenames = self.filenames
-        mytargets = self.targets
+                if ((scope == 'global' and attribute in global_opts) or 
+                    (scope == 'plot' and attribute not in global_opts)):
+                    value = getattr(package, attribute)
+                    self[attribute] = value
+    def process_configs(self, scope):
+        #### Load variables from configs; scope is 'global' or 'plot'
         configdir = tempfile.mkdtemp()
         sys.path.insert(0, '')
         sys.path.insert(0, configdir)
-        write_to_file(config_string(self.mpl), 
+        write_to_file(config_string(), 
                       joined(configdir, 'default_config.py'))
-        configs = ['default_config.py'] + self.configs
-        rc_name = self.mpl and 'rootplotmplrc' or 'rootplotrc'
+        configs = ['default_config.py']
+        for i, c in enumerate(self.configs):
+            shutil.copy(c, joined(configdir, 'rpconfig%i.py' % i))
+            configs.append('rpconfig%i.py' % i)
+        rc_name = use_mpl and 'rootplotmplrc' or 'rootplotrc'
         rc_path = os.path.expanduser('~/.%s' % rc_name)
         if os.path.exists(rc_path):
             print "Using styles and options from ~/.%s" % rc_name
@@ -131,12 +156,8 @@ class Options(dict):
             configs.insert(1, '%s.py' % rc_name)
         for f in configs:
             myconfig = __import__(f[:-3])
-            self.append_from_package(myconfig)
-        if (self.filenames and myfilenames) or (self.targets and mytargets):
-            raise TypeError("files and targets cannot be specified both on "
-                            "the command line and in a configuration file")
-        if not self.filenames: self.filenames = myfilenames
-        if not self.targets: self.targets = mytargets
+            self.append_from_package(myconfig, scope)
+        self.clean_targets()
         shutil.rmtree(configdir)
 
 
@@ -179,11 +200,10 @@ mpl:::##    http://matplotlib.sourceforge.net/users/customizing.html
 ## You can specify the files to run on through the 'filenames' variable rather
 ## than entering them at the command-line, for example:
 ## filenames = ['histTTbar.root', 'histZmumu.root']
-filenames = []
 
 ## Likewise, you can specify target histograms or directories here rather than 
-## on the command-line.  If empty, the files will be used as targets.
-targets = []
+## on the command-line, for example:
+## targets = ['barrel/15to20', 'barrel/20to30']
 
 ## You might also want to specify fancy labels for the legend here rather 
 ## than on the command-line:
@@ -196,9 +216,10 @@ mpl:::## legend_entries = [r'$\bar{t}t$', r'$Z\rightarrow\mu\mu$']
 ## Leave these lists empty to have them automatically filled according to the
 ## command-line options.  Any list that is filled must be at least as long
 ## as the number of targets or it will throw an error.
-line_colors = []          # normally filled by options.colors
-fill_colors = []          # normally filled by options.colors
-marker_colors = []        # normally filled by options.colors
+
+line_colors = []                # normally filled by options.colors
+fill_colors = []                # normally filled by options.colors
+marker_colors = []              # normally filled by options.colors
 mpl:::errorbar_colors = []      # color for bars around the central value
 root::marker_sizes = []         # in pixels
 mpl:::marker_sizes = []         # in points
@@ -425,18 +446,31 @@ html_template=r'''
     </html>
 '''
 """
-config_template = config_template[1:]
 
-multi_call_template = '''calls.append("""
+multi_call_template = '''
+calls.append("""
 %s
 %s
 """)
 '''
 
-allplots_template = '''try:
+allplots_template = '''
+## This file contains all the necessary calls to the rootplot API to produce
+## the same set of plots that were created from the command-line.
+
+## You can use this file to intercept the objects and manipulate them before
+## the figure is saved, making any custom changes that are not possible from
+## the command-line.
+
+## 'objects' is a python dictionary containing all the elements used in the
+## plot, including 'hists', 'legend', etc.
+##   ex: objects['hists'] returns a list of histograms
+
+try:
+  ## the normal way to import rootplot
   from rootplot import plot, plotmpl
 except ImportError:
-  ## check for a CMSSW installation of rootplot
+  ## special import for CMSSW installations of rootplot
   from PhysicsTools.PythonAnalysis.rootplot import plot, plotmpl
 
 import os
@@ -445,13 +479,19 @@ os.chdir('..')  # return to the directory with the ROOT files
 %(call_statements)s
 '''
 
-allplots_multi_template = '''try:
+allplots_multi_template = '''
+## This file is the same as allplots.py, except that it uses multiprocessing
+## to make better use of machines with multiple cores
+
+try:
+  ## the normal way to import rootplot
   from rootplot import plot, plotmpl
   from rootplot.core import report_progress
 except ImportError:
-  ## check for a CMSSW installation of rootplot
+  ## special import for CMSSW installations of rootplot
   from PhysicsTools.PythonAnalysis.rootplot import plot, plotmpl
   from PhysicsTools.PythonAnalysis.rootplot.core import report_progress
+import ROOT
 import multiprocessing as multi
 from Queue import Empty
 
@@ -489,6 +529,11 @@ report_progress(len(calls), len(calls), '%(output)s', '%(ext)s')
 print ''
 '''
 
+## remove leading blank lines from the templates
+for key, value in globals().items():
+    if 'template' in key:
+        globals()[key] = value[1:]
+
 ##############################################################################
 ######## The Command-Line Interface ##########################################
 
@@ -498,17 +543,18 @@ def cli_rootplotmpl():
 
     It is invoked from the command-line as ``rootplotmpl``.
     """
-    cli_rootplot(use_mpl=True)
+    global use_mpl
+    use_mpl = True
+    cli_rootplot()
     
-def cli_rootplot(use_mpl=False):
+def cli_rootplot():
     """
     An application for plotting histograms from a ROOT file.
 
     It is invoked from the command-line as ``rootplot``.
     """
-    options = parse_arguments(sys.argv[1:], use_mpl)
+    options = parse_arguments(sys.argv[1:])
     optdiff = options.kwarg_list()
-    optdiff['mpl'] = use_mpl
     if options.debug:
         rootplot(*options.arguments(), **optdiff)
     else:
@@ -543,7 +589,8 @@ def plotmpl(*args, **kwargs):
 
     Returns the tuple (*figure*, *axeses*, *stack*, *hists*, *plotpath*).
     """
-    kwargs['mpl'] = True
+    global use_mpl
+    use_mpl = True
     return plot(*args, **kwargs)
 
 def plot(*args, **kwargs):
@@ -568,7 +615,7 @@ def plot(*args, **kwargs):
     Returns the tuple (*canvas*, *pads*, *stack*, *hists*, *plotpath*).
     """
     hists, options = initialize_hists(args, kwargs)
-    if options.mpl:
+    if use_mpl:
         return plot_hists_mpl(hists, options)
     else:
         return plot_hists_root(hists, options)
@@ -600,7 +647,8 @@ def rootplotmpl(*args, **kwargs):
     *kwargs*, which can accept any of the options available to
     :mod:`rootplotmpl` at the command-line.
     """
-    kwargs['mpl'] = True
+    global use_mpl
+    use_mpl = True
     rootplot(args, kwargs)
 
 def rootplot(*args, **kwargs):
@@ -630,20 +678,18 @@ def rootplot(*args, **kwargs):
     *kwargs*, which can accept any of the options available to
     :mod:`rootplot` at the command-line.
     """
-    use_mpl = kwargs.pop('mpl', False)
-    funcname = use_mpl and 'rootplotmpl' or 'rootplot'
     if 'config' in kwargs:
-        write_config(use_mpl)
-    options = fill_options(args, kwargs, use_mpl)
+        write_config()
+    options = fill_options(args, kwargs, scope='global')
     nfiles = len(options.filenames)
     ntargets = len(options.targets)
     if nfiles < 1:
         raise TypeError("%s takes at least 1 filename argument (0 given)" %
-                        funcname)
+                        prog)
     elif ntargets > 0 and nfiles > 1:
         raise TypeError("%s cannot accept targets (%i given) when "
                         "multiple files are specified (%i given)" %
-                        (funcname, ntargets, nfiles))
+                        (prog, ntargets, nfiles))
     rootfiles = [RootFile(filename) for filename in options.filenames]
     #### Create the output directory structure
     if not options.noclean and os.path.exists(options.output):
@@ -652,21 +698,19 @@ def rootplot(*args, **kwargs):
         if not os.path.exists(joined(options.output, path)):
             os.makedirs(joined(options.output, path))
     #### Loop over plots to make, building the necessary calls
-    if nfiles > 1 or ntargets > 1:
-        options.draw2D = None
     plotargs = get_plot_inputs(rootfiles, options)
     call_lists = []
+    ndigits = int(math.log10(len(plotargs))) + 1
     for i, (filenames, targets) in enumerate(plotargs):
         argstring = ', '.join(["'%s'" % x for x in (filenames + targets +
                                                     options.configs)])
         reduced_kwargs = dict(kwargs)
         for key, value in reduced_kwargs.items():
-            if key in ['debug', 'path', 'processors', 'merge',
-                       'noclean', 'nopy']:
+            if key in global_opts:
                 del reduced_kwargs[key]
             elif type(value) is str:
                 reduced_kwargs[key] = "'%s'" % value
-        if reduced_kwargs.has_key('numbering'):
+        if kwargs.has_key('numbering'):
             reduced_kwargs['numbering'] = i + 1
         optstring = ', '.join(['%s=%s' % (key, value)
                                for key, value in reduced_kwargs.items()])
@@ -674,6 +718,10 @@ def rootplot(*args, **kwargs):
             argstring = "%s, %s" % (argstring, optstring)
         plotpath, title, legentries = get_plotpath(filenames, targets)
         savepath = joined(options.output, plotpath)
+        if 'numbering' in reduced_kwargs:
+            dirs = savepath.split('/')
+            dirs[-1] = str(i + 1).zfill(ndigits) + dirs[-1]
+            savepath = '/'.join(dirs)
         call_vars = {'argstring' : argstring, 'ext' : options.ext,
                      'savepath' : savepath}
         if use_mpl:
@@ -693,8 +741,8 @@ def rootplot(*args, **kwargs):
     ext = options.ext
     output = options.output
     processors = options.processors
-    call_statements = '\n\n' + '\n'.join([plotcall + '\n' + savecall 
-                                          for plotcall, savecall in call_lists])
+    call_statements = '\n\n'.join([plotcall + '\n' + savecall 
+                                   for plotcall, savecall in call_lists])
     allplots_script = allplots_template % locals()
     call_statements = "".join([multi_call_template % (plotcall, savecall) 
                                for plotcall, savecall in call_lists])
@@ -723,36 +771,15 @@ def rootplot(*args, **kwargs):
     if options.ext in ['png', 'gif', 'svg']:
         print "Writing html index files..."
         width, height = options.size
-        if options.mpl:
+        if use_mpl:
             width, height = [x * options.dpi for x in options.size]
-        if (os.path.exists(joined(options.output, 'plot.' + options.ext))
-            and len(plotargs) == 1):
-            #### targets were histograms
-            make_html_index(options.output, [], ['plot'], options.ext,
+        for path, dirs, files in os.walk(options.output):
+            make_html_index(path, dirs, files, options.ext,
                             options.html_template, options.ncolumns_html,
                             width, height)
-        else:
-            for path, folders, objects in walk_rootfile('', rootfiles[0],
-                                                        options):
-                plotpath = joined(options.output, path)
-                folders = [f for f in folders
-                           if os.path.exists(joined(plotpath, f))]
-                while plotpath != options.output[:-1]:
-                    if os.path.exists(plotpath):
-                        make_html_index(plotpath, folders, objects, options.ext,
-                                        options.html_template,
-                                        options.ncolumns_html,
-                                        width, height)
-                        break
-                    plotpath = joined(options.output,
-                                      '/'.join(plotpath.split('/')[2:]))
     if options.merge:
-        if (os.path.exists(joined(options.output, 'plot.pdf'))
-            and len(plotargs) == 1):
-            merge_pdf(rootfiles[0], options,
-                      joined(options.output, 'plot.pdf'))
-        else:
-            merge_pdf(rootfiles[0], options)
+        merge_pdf(options)
+
 
 ##############################################################################
 ######## Implementation ######################################################
@@ -777,7 +804,7 @@ def option_diff(default, modified):
             diff[key] = modified_val
     return diff
 
-def config_string(use_mpl):
+def config_string():
     s = config_template
     if use_mpl:
         s = re.sub('root::.*\n', '', s)
@@ -789,18 +816,18 @@ def config_string(use_mpl):
         s = s.replace('%prog', 'rootplot')
     return s
 
-def write_config(use_mpl):
+def write_config():
     if use_mpl:
         filename = 'rootplotmpl_config.py'
     else:
         filename = 'rootplot_config.py'
     f = open(filename, 'w')
-    f.write(config_string(use_mpl))
+    f.write(config_string())
     f.close()
     print "Wrote %s to the current directory" % filename
     sys.exit(0)
 
-def add_from_config_files(options, configs, use_mpl):
+def add_from_config_files(options, configs):
     def append_to_options(config, options):
         for attribute in dir(config):
             if '__' not in attribute:
@@ -810,7 +837,7 @@ def add_from_config_files(options, configs, use_mpl):
     sys.path.insert(0, '')
     sys.path.insert(0, configdir)
     f = open(joined(configdir, 'default_config.py'), 'w')
-    f.write(config_string(use_mpl))
+    f.write(config_string())
     f.close()
     import default_config
     append_to_options(default_config, options)
@@ -878,11 +905,11 @@ def get_plot_inputs(files, options):
     else:
         return [(options.filenames, list(t)) for t in target_lists]
 
-def fill_options(args, kwargs, use_mpl):
-    options = parse_arguments(args, use_mpl)
+def fill_options(args, kwargs, scope):
+    options = parse_arguments(args, scope=scope)
     for key, value in kwargs.items():
         options[key] = value
-    options.process_configs()
+    options.process_configs(scope=scope)
     options.size = parse_size(options.size)
     options.split = options.ratio_split or options.efficiency_split
     options.ratio = (options.ratio or options.efficiency or
@@ -1097,7 +1124,7 @@ def plot_hists_mpl(hists, options):
                     label_rotation=options.xlabel_rotation,
                     label_alignment=options.xlabel_alignment)
         if 'barh' not in options.plot_styles:
-            axes.set_xlim(hist.xedges[0], hist.xedges[-1])
+            axes.set_xlim(refhist.xedges[0], refhist.xedges[-1])
         if options.logy:
             my_min = fullstack.min(threshold=1.1e-10)
             rounded_min = 1e100
@@ -1179,9 +1206,7 @@ def plot_hists_mpl(hists, options):
     return fig, objects
 
 def initialize_hists(args, kwargs):
-    use_mpl = kwargs.get('mpl', False)
-    funcname = use_mpl and 'plotmpl' or 'plot'
-    options = fill_options(args, kwargs, use_mpl)
+    options = fill_options(args, kwargs, scope='plot')
     if use_mpl:
         load_matplotlib(options.ext)
     nfiles = len(options.filenames)
@@ -1325,16 +1350,12 @@ def report_progress(counter, nplots, output, ext, divisor=1):
               (counter, nplots, output, ext)),
         sys.stdout.flush()
 
-def merge_pdf(rootfile, options, plotpath=None):
+def merge_pdf(options):
     #### Merge together all the produced plots into one pdf file.
     destination = joined(options.output, 'allplots.pdf')
     paths = []
-    if plotpath:
-        paths.append(plotpath)
-    else:
-        for path, folders, objects in walk_rootfile('', rootfile, options):
-            for obj in objects:
-                paths.append(joined(options.output, path, obj + '.pdf'))
+    for path, dirs, files in os.walk(options.output):
+        paths += [joined(path, x) for x in files if x.endswith('.pdf')]
     if not paths:
         print "No output files, so no merged pdf was made"
         return
@@ -1501,10 +1522,9 @@ def plot_ratio_mpl(axes, hists, options):
     axes.yaxis.set_label_position('right')
     axes.yaxis.label.set_rotation(-90)
 
-def make_html_index(path, dirs, objects, filetype, template, ncolumns,
+def make_html_index(path, dirs, files, filetype, template, ncolumns, 
                     width, height):
-    objects = [x for x in objects 
-               if os.path.exists(joined(path, '%s.%s' % (x, filetype)))]
+    files = [x for x in files if x.endswith(filetype)]
     output_file = open(joined(path, 'index.html'), 'w')
     previous_dirs = [x for x in path.split('/') if x]
     ndirs = len(previous_dirs)
@@ -1523,13 +1543,12 @@ def make_html_index(path, dirs, objects, filetype, template, ncolumns,
         imgtemplate += '<img src="%(plot)s" height=%(height)i width=%(width)i>'
     imgtemplate += '</a></a>'
     plots = '\n'
-    for obj in objects:
-        plot = '%s.%s' % (obj, filetype)
+    for plot in files:
         plots += imgtemplate % locals() + '\n'
     plots = re.sub('((\\n<a.*){%i})' % ncolumns, r'\1<br>', plots)
     output_file.write(template % locals())
     output_file.close()
-
+    
 def parse_range(xedges, expression):
     #### Returns the indices of the low and high bins indicated in expression.
     closest = lambda l,y: l.index(min(l, key=lambda x:abs(x-y)))
@@ -1643,29 +1662,29 @@ def process_options(options):
         except TypeError:
             return [eval(objtype)(obj) for i in range(nhists)]
     nhists = options.nhists
-    plotpath, title, legentries = get_plotpath(options.filenames,
-                                               options.targets)
-    options.plotpath = plotpath
-    if not options.title:
-        options.title = title
-    if not options.legend_entries:
-        options.legend_entries = legentries
-    options.legend_entries = comma_separator(options.legend_entries,
-                                             str, nhists)
+    if options.targets:
+        plotpath, title, legentries = get_plotpath(options.filenames,
+                                                   options.targets)
+        options.plotpath = plotpath
+        if not options.title:
+            options.title = title
+        if not options.legend_entries:
+            options.legend_entries = legentries
+        options.legend_entries = comma_separator(options.legend_entries,
+                                                 str, nhists)
     options.scale = comma_separator(options.scale, float, nhists)
     if options.efficiency_split: options.ratio_max = 1.
     if nhists > 1: options.draw2D = None
-    if options.mpl:
-        if options.bar: options.mpl = 'bar'
-        elif options.barh: options.mpl = 'barh'
-        elif options.barcluster: options.mpl = 'barcluster'
-        elif options.errorbar: options.mpl = 'errorbar'
-        elif options.hist: options.mpl = 'hist'
-        elif options.histfill: options.mpl = 'histfill'
-        if options.stack: options.mpl = 'stack'
-        if type(options.mpl) is not str:
-            options.mpl = 'histfill'
-    if not options.markers and options.mpl:
+    if use_mpl:
+        plot_style = 'histfill'
+        if options.bar: plot_style = 'bar'
+        elif options.barh: plot_style = 'barh'
+        elif options.barcluster: plot_style = 'barcluster'
+        elif options.errorbar: plot_style = 'errorbar'
+        elif options.hist: plot_style = 'hist'
+        elif options.histfill: plot_style = 'histfill'
+        if options.stack: plot_style = 'stack'
+    if not options.markers and use_mpl:
         options.marker_styles = ['o' for i in xrange(nhists)]
     if not options.line_colors:
         options.line_colors = options.colors
@@ -1673,11 +1692,11 @@ def process_options(options):
         options.fill_colors = options.colors
     if not options.marker_colors:
         options.marker_colors = options.colors
-    if options.mpl:
+    if use_mpl:
         if not options.line_styles:
             options.line_styles = ['solid' for i in xrange(nhists)]
         if not options.plot_styles:
-            options.plot_styles = [options.mpl for i in xrange(nhists)]
+            options.plot_styles = [plot_style for i in xrange(nhists)]
         if not options.errorbar_colors:
             options.errorbar_colors = [None for i in xrange(nhists)]
         if not options.alphas:
@@ -1699,7 +1718,7 @@ def process_options(options):
                                          for i in xrange(nhists)]
     if not options.marker_sizes:
         if options.markers:
-            if options.mpl: size = mpl.rcParams['lines.markersize']
+            if use_mpl: size = mpl.rcParams['lines.markersize']
             else: size = ROOT.gStyle.GetMarkerSize()
         else:
             size = 0
@@ -1708,14 +1727,14 @@ def process_options(options):
         i = options.data - 1
         options.line_colors[i] = options.data_color
         options.fill_colors[i] = options.data_color
-        if options.mpl:
+        if use_mpl:
             options.plot_styles[i] = 'errorbar'
         else:
             options.fill_styles[i] = 0
             options.draw_commands[i] = 'e'
         options.marker_styles[i] = options.data_marker
         if not options.marker_sizes[i]:
-            if options.mpl:
+            if use_mpl:
                 options.marker_sizes[i] = mpl.rcParams['lines.markersize']
             else:
                 options.marker_sizes[i] = ROOT.gStyle.GetMarkerSize()
@@ -1724,20 +1743,21 @@ def process_options(options):
     for opt in [x for x in options.keys() if 'colors' in x]:
         try:
             colors = options[opt]
-            options[opt] = [parse_color(x, not options.mpl) for x in colors]
+            options[opt] = [parse_color(x, not use_mpl) for x in colors]
         except AttributeError:
             pass
-    #### Apply extra options based on hist name
-    plotname = os.path.basename(options.plotpath)
-    for option, value, regexes in options.options_by_histname:
-        for regex in regexes:
-            if re.match(regex, plotname):
-                setattr(options, option, value)
-    #### Final setup
-    if options.logy:
-        if options.ymin <= 0:
-            options.ymin = None
-        options.top_padding_factor = options.top_padding_factor_log
+    if options.targets:
+        #### Apply extra options based on hist name
+        plotname = os.path.basename(options.plotpath)
+        for option, value, regexes in options.options_by_histname:
+            for regex in regexes:
+                if re.match(regex, plotname):
+                    setattr(options, option, value)
+        #### Final setup
+        if options.logy:
+            if options.ymin <= 0:
+                options.ymin = None
+            options.top_padding_factor = options.top_padding_factor_log
 
 def cartesian_product(*args, **kwds):
     # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
@@ -1749,7 +1769,7 @@ def cartesian_product(*args, **kwds):
     for prod in result:
         yield tuple(prod)
 
-def parse_arguments(argv, use_mpl=False):
+def parse_arguments(argv, scope='global'):
     if use_mpl:
         import matplotlib as mpl
         figsize = 'x'.join([str(x) for x in mpl.rcParams['figure.figsize']])
@@ -1811,7 +1831,8 @@ def parse_arguments(argv, use_mpl=False):
     addopt(g_output, '--output', metadefault='plots', 
            help="name of output directory")
     addopt(g_output, '--numbering', action='store_true', default=False,
-           help="add a page number in the upper right of each plot")
+           help="print page numbers on images and prepend them to file names; "
+           "numbering will respect the order of objects in the ROOT file")
     addopt(g_output, '--size', metadefault=figsize,
            root=opt(help="set the canvas size to 'width x height' in pixels"),
            mpl=opt(help="set the canvas size to 'width x height' in inches"))
@@ -1828,14 +1849,14 @@ def parse_arguments(argv, use_mpl=False):
            help="normalize all histograms by VAL, or by individual values "
            "if VAL is a comma-separated list")
     addopt(g_hist, '--normalize', metavar='NUM', type='int', default=0,
-           help="normalize to the NUMth file/target (starting with 1)")
+           help="normalize to the NUMth target (starting with 1)")
     addopt(g_hist, '--norm-range', metavar='LOWxHIGH',
            help="only use the specified data range in determining "
            "the normalization")
     addopt(g_hist, '--rebin', metavar="N", type=int,
            help="group together bins in sets of N")
     addopt(g_hist, '--ratio', type='int', default=0, metavar='NUM',
-           help="divide histograms by the NUMth file/target (starting from 1)")
+           help="divide histograms by the NUMth target (starting from 1)")
     addopt(g_hist, '--ratio-split', type='int', default=0, metavar='NUM',
            help="same as --ratio, but split the figure in two, displaying "
            "the normal figure on top and the ratio on the bottom")
@@ -1852,11 +1873,12 @@ def parse_arguments(argv, use_mpl=False):
                "errorbars, or 'hist' to make sure no errorbars appear")
     addopt(g_style, '--draw2D',
            root=opt(metadefault='box',
-                    help="argument to pass to TH2::Draw (2D hists only "
-                    "drawn for single files); set "
+                    help="argument to pass to TH2::Draw (ignored if multiple "
+                    "targets specified); set "
                     'to "" to turn off 2D drawing'),
            mpl=opt(metadefault='box', 
-                   help="command to use for plotting 2D hists; "
+                   help="command to use for plotting 2D hists; (ignored if "
+                   "multiple targets specified) "
                    "choose from 'contour', 'col', 'colz', and 'box'")
            )
     if not use_mpl:
@@ -1882,11 +1904,11 @@ def parse_arguments(argv, use_mpl=False):
     addopt(g_style, '--xerr', action="store_true", default=False,
            help="show width of bins")
     addopt(g_style, '--data', type='int', default=0, metavar='NUM',
-           root=opt(help="treat the NUMth file/target (starting from 1) "
+           root=opt(help="treat the NUMth target (starting from 1) "
                     "specially, drawing it as black datapoints; to achieve "
                     "a classic data vs. MC plot, try this along with "
                     "--stack and --fill"),
-           mpl=opt(help="treat the NUMth file/target (starting from 1) "
+           mpl=opt(help="treat the NUMth target (starting from 1) "
                    "specially, drawing it as black datapoints; to achieve "
                    "a classic data vs. MC plot, try this along with --stack"))
     addopt(g_style, '--xmax', type='float', default=None,
@@ -1937,13 +1959,8 @@ def parse_arguments(argv, use_mpl=False):
            help="display underflow content in the lowest bin")
     #### Do immediate processing of arguments
     options, arguments = parser.parse_args(list(argv))
-    options = Options(options)
-    options.filenames = [x for x in arguments if x.endswith('.root')]
-    options.configs   = [x for x in arguments if x.endswith('.py')  ]
-    options.targets   = [x for x in arguments if not (x.endswith('.root') or
-                                                      x.endswith('.py'))]
-    options.mpl = use_mpl
-    options.replace = []
+    options = Options(options, arguments, scope=scope)
+    options.replace = [] # must have default in case not using mpl
     if options.processors == 1 or options.ext == 'C':
         global use_multiprocessing
         use_multiprocessing = False
